package com.ferri.core.utils;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

import com.ferri.core.utils.exception.GenericClassNotFound;
import com.ferri.core.utils.exception.GenericTypeNotFound;

/**
 *
 */
public abstract class ClazzUtils {

	/**
	 * Obtain the expected type from the generic parent.<br>
	 * The first type matched is returned.
	 *
	 * @param genericClazz
	 *            the given class that inherit from a generic parent clazz.
	 * @param expectedGenericParent
	 *            the generic parent class to match.
	 * @param expectedType
	 *            the type to match
	 * @return the expected type from the given parameters *
	 * @throws GenericClassNotFound
	 *             if the given class dont have the expected parent class
	 * @throws GenericTypeNotFound
	 *             if the given type is not found
	 */
	public static Type getEntityType(Class<?> genericClazz, Class<?> expectedGenericParent) throws GenericClassNotFound, GenericTypeNotFound {
		return getEntityType(genericClazz, expectedGenericParent, 1);
	}

	/**
	 * Obtain the expected type from the generic parent
	 *
	 * @param genericClazz
	 *            the given class that inherit from a generic parent clazz.
	 * @param expectedGenericParent
	 *            the generic parent class to match.
	 * @param expectedType
	 *            the type to match
	 * @param expectedIndex
	 *            the index of the type to match if has more than one
	 * @return the expected type from the given parameters
	 * @throws GenericClassNotFound
	 *             if the given class dont have the expected parent class
	 * @throws GenericTypeNotFound
	 *             if the given type is not found
	 */
	public static Type getEntityType(Class<?> genericClazz, Class<?> expectedGenericParent, Integer expectedIndex) throws GenericClassNotFound,
			GenericTypeNotFound {

		if (genericClazz == null) {
			throw new IllegalArgumentException("'genericClazz' can not be null");
		}

		if (expectedGenericParent == null) {
			throw new IllegalArgumentException("'expectedGenericParent' can not be null");
		}

		// if (expectedType == null) {
		// throw new IllegalArgumentException("'expectedType' can not be null");
		// }

		Type type = null;

		Class<?> goTopClass = getTopClass(genericClazz, expectedGenericParent);

		Type genericSuperclass = goTopClass.getGenericSuperclass();
		if (genericSuperclass instanceof ParameterizedType) {
			ParameterizedType parameterizedType = (ParameterizedType) genericSuperclass;
			// type = parameterizedType.getActualTypeArguments()[0];
			type = getSelectedType(parameterizedType.getActualTypeArguments(), expectedIndex);
		}

		if (type == null) {
			throw new GenericClassNotFound("the given type index [" + expectedIndex + "] was not found in generic parent class ["
					+ expectedGenericParent + "] of this[" + genericClazz + "]");
		}

		return type;
	}

	/**
	 * Obtain the expecte type from the given array types<br>
	 * When the type have duplicates an index is necesary.
	 *
	 * @param types
	 *            the allowed array type
	 * @param expectedType
	 *            the type to match.
	 * @param paramIndex
	 *            the index start at 1
	 * @return the type expeced from the list.
	 * @throws GenericTypeNotFound
	 *             if the given type is not found
	 */
	public static Type getSelectedType(Type[] types, Integer paramIndex) throws GenericTypeNotFound {
		Integer expectedIndex = 1;

		if (types == null) {
			throw new IllegalArgumentException("'types' can not be null");
		}

		if (paramIndex == null) {
			throw new IllegalArgumentException("'paramIndex' can not be null");
		}

		if (paramIndex != null) {
			expectedIndex = paramIndex - 1;
		}

		if (expectedIndex >= types.length || expectedIndex < 0) {
			throw new IndexOutOfBoundsException("Index types: " + paramIndex + ", Size: " + types.length);
		}

		Type ret = types[expectedIndex];

		if (ret == null) {
			throw new GenericTypeNotFound("the given paramIndex [" + paramIndex + "] with the index[" + expectedIndex
					+ "] is not found in the types array ");
		}

		return ret;
	}

	/**
	 * Obtain the expected parent from the given class instance
	 *
	 * @param currentClazz
	 *            the given class instance
	 * @param expectedParent
	 *            the parent to match
	 *
	 * @return the parent class definition.
	 */
	public static Class<?> getTopClass(Class<?> currentClazz, Class<?> expectedParent) {

		if (currentClazz == null) {
			throw new IllegalArgumentException("'currentClazz' can not be null");
		}

		if (expectedParent == null) {
			throw new IllegalArgumentException("'expectedParent' can not be null");
		}

		Class<?> expectedClazz = currentClazz;

		while (!expectedClazz.equals(expectedParent) && expectedClazz.getSuperclass() != null
				&& !expectedClazz.getSuperclass().equals(expectedParent)) {

			expectedClazz = expectedClazz.getSuperclass();
		}

		return expectedClazz;
	}

}
