/**
 *
 */
package com.ferri.core.vo.commons.converters;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import com.ferri.core.behaviors.Initializable;
import com.ferri.core.utils.ClazzUtils;
import com.ferri.core.utils.exception.GenericClassNotFound;
import com.ferri.core.utils.exception.GenericTypeNotFound;
import com.ferri.core.vo.AbstractBase;
import com.ferri.core.vo.commons.BaseDTO;
import com.ferri.core.vo.commons.BaseVO;

/**
 * @author dferri
 *
 */
public abstract class AbstractConverter<DTO extends BaseDTO, VO extends BaseVO> implements Initializable {

	private static final String CONVERT_DTO_2_VO = "convertDTO2VO";

	private Logger logger = Logger.getLogger(this.getClass());

	/**
	 * The Class instance of this <code>DispatchAction</code> class.
	 */
	@SuppressWarnings("unchecked")
	protected Class<? extends AbstractConverter> clazz = this.getClass();

	/**
	 * The set of Method objects we have introspected for this class, keyed by method name. This collection is populated as
	 * different methods are called, so that introspection needs to occur only once per method name.
	 */
	private HashMap<String, Method> methods = new HashMap<String, Method>();

	private BeansConverter beansConverter = null;

	private Class<DTO> classTypeDTO = null;

	private Class<VO> classTypeVO = null;

	/**
	 * Convert instance DTO to instance VO
	 * @param dto DTO
	 * @return VO
	 */
	public abstract VO convertDTO2VO(DTO dto);

	/**
	 *  Convert instance VO to instance DTO
	 * @param vo VO
	 * @return DTO
	 */
	public abstract DTO convertVO2DTO(VO vo);

	public final Class<DTO> getClassTypeDTO() {
		return classTypeDTO;
	}

	public final Class<VO> getClassTypeVO() {
		return classTypeVO;
	}

	// ---- BeanConvert Uncheck {
	@SuppressWarnings("unchecked")
	VO internalWarpDTO2VO(BaseDTO baseDTO) {
		return convertDTO2VO((DTO) baseDTO);
	}

	@SuppressWarnings("unchecked")
	VO internalWarpDTO2VO(BaseDTO baseDTO, String methodName) throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
		return (VO) executeMethod(baseDTO, CONVERT_DTO_2_VO, methodName);
	}

	@SuppressWarnings("unchecked")
	DTO internalWarpVO2DTO(BaseVO baseVO) {
		return convertVO2DTO((VO) baseVO);
	}

	@SuppressWarnings("unchecked")
	DTO internalWarpVO2DTO(BaseVO baseVO, String methodName) throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
		return (DTO) executeMethod(baseVO, CONVERT_DTO_2_VO, methodName);
	}

	// ---- BeanConvert Uncheck }

	public Collection<VO> convertCollectionDTO2VO(Collection<BaseDTO> colDTO) {
		Collection<VO> colVO = new ArrayList<VO>();

		if (colDTO != null && !colVO.isEmpty()) {
			for (BaseDTO dto : colDTO) {
				colVO.add(internalWarpDTO2VO(dto));
			}
		}

		return colVO;
	}

	public Collection<VO> convertCollectionDTO2VO(Collection<BaseDTO> colDTO, String methodName) throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
		Collection<VO> colVO = new ArrayList<VO>();

		if (colDTO != null && !colVO.isEmpty()) {
			for (BaseDTO dto : colDTO) {
				colVO.add(internalWarpDTO2VO(dto, methodName));
			}
		}

		return colVO;
	}

	public Collection<DTO> convertCollectionVO2DTO(Collection<BaseVO> colVO) {
		Collection<DTO> colDTO = new ArrayList<DTO>();

		if (colVO != null && !colVO.isEmpty()) {
			for (BaseVO vo : colVO) {
				colDTO.add(internalWarpVO2DTO(vo));
			}
		}

		return colDTO;
	}

	public Collection<DTO> convertCollectionVO2DTO(Collection<BaseVO> colVO, String methodName) throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
		Collection<DTO> colDTO = new ArrayList<DTO>();

		if (colVO != null && !colVO.isEmpty()) {
			for (BaseVO vo : colVO) {
				colDTO.add(internalWarpVO2DTO(vo, methodName));
			}
		}

		return colDTO;
	}

	public void registerInConverters(Map<Class<?>, AbstractConverter<? extends BaseDTO, ? extends BaseVO>> converters) {
		converters.put(getClassTypeDTO(), this);
		converters.put(getClassTypeVO(), this);
	}

	public void deregisterInConverters(Map<Class<?>, AbstractConverter<? extends BaseDTO, ? extends BaseVO>> converters) {
		converters.remove(getClassTypeDTO());
		converters.remove(getClassTypeVO());
	}

	/**
	 * @param abstractBase
	 * @param baseMethodName
	 * @param methodName
	 * @return
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 * @throws IllegalAccessException
	 */
	private AbstractBase executeMethod(AbstractBase abstractBase, String baseMethodName, String methodName) throws InvocationTargetException,
			NoSuchMethodException, IllegalAccessException {
		AbstractBase ret = null;
		String methodPattern = getMethodPattern(baseMethodName, methodName);
		try {
			Method method = getMethod(methodPattern, abstractBase.getClass());
			ret = (AbstractBase) method.invoke(this, abstractBase);
		} catch (NoSuchMethodException e) {
			getLogger().error("method not found", e);
			throw e;
		} catch (IllegalArgumentException e) {
			getLogger().error("method found but arguments does not match", e);
			throw e;
		} catch (IllegalAccessException e) {
			getLogger().error("method revise the permition", e);
			throw e;
		} catch (InvocationTargetException e) {
			throw e;
		}
		return ret;
	}

	/**
	 * @throws GenericClassNotFound
	 *             if the given class dont have the expected parent class
	 * @throws GenericTypeNotFound
	 *             if the given type is not found
	 * @see com.ferri.core.behaviors.Initializable#initialize()
	 */
	public void initialize() throws GenericClassNotFound, GenericTypeNotFound {
		initiateObjectsType();
	}

	/**
	 * Initialize casters objects
	 *
	 * @throws GenericClassNotFound
	 *             if the given class dont have the expected parent class
	 * @throws GenericTypeNotFound
	 *             if the given type is not found
	 */
	@SuppressWarnings("unchecked")
	private void initiateObjectsType() throws GenericClassNotFound, GenericTypeNotFound {
		if (getClassTypeDTO() == null) {
			classTypeDTO = (Class<DTO>) ClazzUtils.getEntityType(this.getClass(), AbstractConverter.class, 1);
		}

		if (getClassTypeVO() == null) {
			classTypeVO = (Class<VO>) ClazzUtils.getEntityType(this.getClass(), AbstractConverter.class, 2);
		}
	}

	/**
	 * Introspect the current class to identify a method of the specified name that accepts the same parameter types as the
	 * <code>execute</code> method does.
	 *
	 * @param name
	 *            Name of the method to be introspected
	 * @param arg
	 * @return The method with the specified name.
	 * @throws NoSuchMethodException
	 *             if no such method can be found
	 */
	protected Method getMethod(String name, Class<?> arg) throws NoSuchMethodException {
		synchronized (methods) {
			Method method = methods.get(name);

			if (method == null) {
				method = clazz.getMethod(name, new Class<?>[] { arg });
				methods.put(name, method);
			}

			return (method);
		}
	}

	private String getMethodPattern(String prefix, String methodName) {
		if (prefix == null || methodName == null) {
			throw new IllegalArgumentException("the method name or prefix is null");
		}
		return prefix + "_" + methodName;
	}

	/**
	 * @return the beansConverter
	 */
	public BeansConverter getBeansConverter() {
		return beansConverter;
	}

	/**
	 * @param beansConverter
	 *            the beansConverter to set
	 */
	public void setBeansConverter(BeansConverter beansConverter) {
		this.beansConverter = beansConverter;
	}

	/**
	 * @return the logger
	 */
	protected Logger getLogger() {
		return logger;
	}

}
