package com.ferri.core.vo.commons.converters;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.hibernate.proxy.HibernateProxyHelper;

import com.ferri.core.behaviors.Initializable;
import com.ferri.core.vo.AbstractBase;
import com.ferri.core.vo.commons.BaseDTO;
import com.ferri.core.vo.commons.BaseVO;
import com.ferri.core.vo.commons.converters.exceptions.ConverterNotFoundException;

/**
 * @author dferri
 */
public class BeansConverter implements Initializable {

	public int TYPE_DTO = 1;
	public int TYPE_VO = 2;

	private Logger logger = Logger.getLogger(BeansConverter.class);
	private Map<Class<?>, AbstractConverter<?, ?>> converters = null;

	private Collection<AbstractConverter<?, ?>> convertersCollection = null;

	public BeansConverter() {
		converters = new HashMap<Class<?>, AbstractConverter<?, ?>>();
	}

	public BeansConverter(Collection<AbstractConverter<?, ?>> convertersCollection) {
		converters = new HashMap<Class<?>, AbstractConverter<?, ?>>();
		this.convertersCollection = convertersCollection;
		initialize();
	}

	public void initiateConverters() {
		if (getConvertersCollection() != null && !getConvertersCollection().isEmpty()) {
			for (AbstractConverter<?, ?> converter : getConvertersCollection()) {
				converter.registerInConverters(converters);
			}
		}
	}

	public void resetConverters() {
		if (getConvertersCollection() != null && !getConvertersCollection().isEmpty()) {
			for (AbstractConverter<?, ?> converter : getConvertersCollection()) {
				converter.deregisterInConverters(converters);
			}
		}
	}

	public void initialize() {
		initiateConverters();
	}

	/**
	 * Register a custom {@link AbstractConverter} for the specified destination <code>Class</code>, replacing any previously
	 * registered Converter.
	 *
	 * @param converter
	 *            Converter to be registered
	 * @param clazz
	 *            Destination class for conversions performed by this AbstractConverter
	 */
	public void register(Class<?> clazz, AbstractConverter<BaseDTO, BaseVO> converter) {
		getConverters().put(clazz, converter);
	}

	/**
	 * Remove any registered {@link AbstractConverter} for the specified destination <code>Class</code>.
	 *
	 * @param clazz
	 *            Class for which to remove a registered AbstractConverter
	 */
	public void deregister(Class<?> clazz) {
		getConverters().remove(clazz);
	}

	/**
	 * Look up and return any registered {@link AbstractConverter} for the specified destination class; if there is no registered
	 * Converter, return <code>null</code>.
	 *
	 * @param clazz
	 *            Class for which to return a registered AbstractConverter
	 * @throws ConverterNotFoundException
	 *             if the converter is not found.
	 */
	private AbstractConverter<?, ?> lookup(Class<?> clazz) throws ConverterNotFoundException {
		AbstractConverter<?, ?> abstractConverter = getConverters().get(clazz);

		if (abstractConverter == null) {
			throw new ConverterNotFoundException("No converter was found for this entity:" + clazz.getName());
		}

		return abstractConverter;
	}

	/**
	 * Look up and return any registered {@link AbstractConverter} for the specified destination class; if there is no registered
	 * Converter, return <code>null</code>.
	 *
	 * @param object
	 *            Object for which to return a registered AbstractConverter
	 * @throws ConverterNotFoundException
	 *             if the converter is not found.
	 */
	private AbstractConverter<?, ?> lookup(Object object) throws ConverterNotFoundException {
		return lookup(translateProxyType(object));
	}

	/**
	 * Retrieve the class type of the given object, looking for a proxy from Hibernate.
	 *
	 * @param object
	 *            the object to determine th class type
	 * @return the Class type expected
	 */
	private Class<?> translateProxyType(Object object) {
		return HibernateProxyHelper.getClassWithoutInitializingProxy(object);
	}

	private AbstractBase convertBase(AbstractBase baseOrigin, String methodName, int type) {
		AbstractBase baseDest = null;

		if (validObject(baseOrigin) && validType(type)) {
			String stringType = getConvertedType(type);
			if (methodName == null) {
				getLogger().debug("Convert '" + baseOrigin + stringType);
			} else {
				getLogger().debug("Convert '" + baseOrigin + stringType + " with method name [" + methodName + "]");
			}

			try {
				AbstractConverter<?, ?> converter = lookup(baseOrigin);
				if (converter != null) {
					if (type == TYPE_DTO) {
						if (methodName == null) {
							baseDest = converter.internalWarpDTO2VO((BaseDTO) baseOrigin);
						} else {
							baseDest = converter.internalWarpDTO2VO((BaseDTO) baseOrigin, methodName);
						}
					} else if (type == TYPE_VO) {
						if (methodName == null) {
							baseDest = converter.internalWarpVO2DTO((BaseVO) baseOrigin);
						} else {
							baseDest = converter.internalWarpVO2DTO((BaseVO) baseOrigin, methodName);
						}
					}
				}
			} catch (ConverterNotFoundException e) {
				getLogger().warn(e.getMessage(), e);
			} catch (InvocationTargetException e) {
				getLogger().warn(e.getMessage(), e);
			} catch (NoSuchMethodException e) {
				getLogger().warn(e.getMessage(), e);
			} catch (IllegalAccessException e) {
				getLogger().warn(e.getMessage(), e);
			}
		}
		return baseDest;
	}

	public BaseVO convert(BaseDTO baseDTO, String methodName) {
		return (BaseVO) convertBase(baseDTO, methodName, TYPE_DTO);
	}

	public BaseDTO convert(BaseVO baseVO, String methodName) {
		return (BaseDTO) convertBase(baseVO, methodName, TYPE_VO);
	}

	public BaseVO convert(BaseDTO baseDTO) {
		return convert(baseDTO, null);
	}

	public BaseDTO convert(BaseVO baseVO) {
		return convert(baseVO, null);
	}

	@SuppressWarnings("unchecked")
	public Collection<? extends AbstractBase> convertCollection(Collection<? extends AbstractBase> collection) {
		return convertCollection(collection, null);
	}

	@SuppressWarnings("unchecked")
	public Collection<? extends AbstractBase> convertCollection(Collection<? extends AbstractBase> collection, String methodName) {
		Collection<? extends AbstractBase> retCol = null;

		if (collection != null && !collection.isEmpty()) {
			Object element = getFirstCollectionObject(collection);
			if (validObject(element)) {
				Class<?> clazzType = translateProxyType(element);
				try {
					AbstractConverter<?, ?> converter = lookup(clazzType);
					if (converter != null) {
						if (element instanceof BaseDTO) {
							getLogger().debug(getConvertedDescription(clazzType, converter.getClassTypeVO()));
							if (methodName != null) {
								retCol = converter.convertCollectionDTO2VO((Collection<BaseDTO>) collection, methodName);
							} else {
								retCol = converter.convertCollectionDTO2VO((Collection<BaseDTO>) collection);
							}
						} else if (element instanceof BaseVO) {
							getLogger().debug("Converting Collection<" + clazzType + "> to Collection<" + converter.getClassTypeDTO() + ">");
							if (methodName != null) {
								retCol = converter.convertCollectionVO2DTO((Collection<BaseVO>) collection, methodName);
							} else {
								retCol = converter.convertCollectionVO2DTO((Collection<BaseVO>) collection);
							}
						} else {
							getLogger().warn("Object element incompatible");
						}
					}
				} catch (ConverterNotFoundException e) {
					getLogger().warn(e.getMessage(), e);
				} catch (InvocationTargetException e) {
					getLogger().warn(e.getMessage(), e);
				} catch (NoSuchMethodException e) {
					getLogger().warn(e.getMessage(), e);
				} catch (IllegalAccessException e) {
					getLogger().warn(e.getMessage(), e);
				}
			}else{
				new ArrayList<AbstractBase>();
			}
		} else {
			retCol = new ArrayList<AbstractBase>();
		}

		return retCol;
	}

	/**
	 * Return a description for the given method invocation.
	 *
	 * @param clazzTypeFrom
	 *            the class incomming to describe
	 * @param clazzTypeTO
	 *            the class outcomming to describe
	 * @return the description
	 */
	private String getConvertedDescription(Class<?> clazzTypeFrom, Class<?> clazzTypeTO) {
		return "Converting Collection<" + clazzTypeFrom + "> to Collection<" + clazzTypeTO + ">";
	}

	private String getConvertedType(int type) {
		if (type == TYPE_VO) {
			return "' to class '<? extends BaseVO>'";
		} else if (type == TYPE_DTO) {
			return "' to class '<? extends BaseDTO>'";
		} else {
			getLogger().debug("' to class 'UNDEFINED_TYPE?'");
			return "' to class 'UNDEFINED_TYPE?'";
		}
	}

	/**
	 * Checks for recognized type
	 *
	 * @param type
	 *            the style
	 * @return
	 */
	private boolean validType(int type) {
		return (type == TYPE_DTO) || (type == TYPE_VO);
	}
	
	/**
	 * Checks for null Objects
	 *
	 * @param type
	 *            the style
	 * @return
	 */
	private boolean validObject(Object obj) {
		return obj != null;
	}

	private Object getFirstCollectionObject(Collection<?> collection) {
		return collection.iterator().next();
	}

	/**
	 * @return the converters
	 */
	private Map<Class<?>, AbstractConverter<?, ?>> getConverters() {
		return converters;
	}

	/**
	 * @return the logger
	 */
	private Logger getLogger() {
		return logger;
	}

	/**
	 * @return the convertersCollection
	 */
	public Collection<AbstractConverter<?, ?>> getConvertersCollection() {
		return convertersCollection;
	}

	/**
	 * @param convertersCollection
	 *            the convertersCollection to set
	 */
	public void setConvertersCollection(Collection<AbstractConverter<?, ?>> convertersCollection) {
		this.convertersCollection = convertersCollection;
	}
}
