package com.ferri.persistence.dao.commons;

import java.io.Serializable;
import java.util.Collection;

import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;

import com.ferri.core.behaviors.Initializable;
import com.ferri.core.vo.commons.BaseVO;

/**
 * @author dferri
 */
public interface GenericBaseDAO<E extends BaseVO, PK extends Serializable> extends Initializable {

	String ORDER_BY = "order by ";
	String SEPARATOR_COMMA = ", ";
	String SEPARATOR_DOT = ".";

	/**
	 * Loads a Entity instance by identifier.
	 *
	 * @return Entity the requested instance.
	 * @param key
	 *            the identifier
	 */
	E load(PK Id);

	/**
	 * Return all objects related to the implementation of this DAO with no filter.
	 *
	 * @return Collection all non filtered objects.
	 */
	Collection<E> loadAll();

	/**
	 * Persist the given transient instance, first assigning a generated identifier. (Or using the current value of the identifier
	 * property if the assigned generator is used.)
	 *
	 * @param entity
	 *            a transient instance of a persistent class.
	 * @return the class identifier.
	 */
	PK create(E entity);

	/**
	 * Persist the given transient collection instance, first assigning a generated identifier. (Or using the current value of the
	 * identifier property if the assigned generator is used.)
	 *
	 * @param entities
	 *            the collection to persist.
	 * @return Collection the created entities.
	 */
	Collection<E> create(final Collection<E> entities);

	/**
	 * Save the persistent state associated with the given identifier. An exception is thrown if there is a persistent instance
	 * with the same identifier in the current session.
	 *
	 * @param entity
	 *            a transient instance containing saved Or Updated state.
	 */
	void save(E entity);

	/**
	 * Update a persistent instance from the datastore. The argument may be an instance associated with the receiving Session or a
	 * transient instance with an identifier associated with existing persistent state.
	 *
	 * @param entity
	 *            the instance to be updated.
	 */
	void update(E entity);

	/**
	 * saveOrUpdate the persistent state associated with the given identifier. An exception is thrown if there is a persistent
	 * instance with the same identifier in the current session.
	 *
	 * @param entity
	 *            a transient instance containing saved Or Updated state.
	 */
	void saveOrUpdate(E entity);

	/**
	 * saveOrUpdate the persistent collection state associated with the given identifier. An exception is thrown if there is a
	 * persistent instance with the same identifier in the current session.
	 *
	 * @param entities
	 *            the collection to saveOrUpdate.
	 */
	void saveOrUpdate(final Collection<E> entities);

	/**
	 * Remove a persistent instance from the datastore. The argument may be an instance associated with the receiving Session or a
	 * transient instance with an identifier associated with existing persistent state.
	 *
	 * @param id
	 *            the instance ID to be removed.
	 */
	void removeByPk(PK id);

	/**
	 * Remove a persistent instance from the datastore. The argument may be an instance associated with the receiving Session or a
	 * transient instance with an identifier associated with existing persistent state.
	 *
	 * @param entity
	 *            the instance to be removed.
	 */
	void remove(E entity);

	/**
	 * Removes all entities in the given <code>entities<code> collection.
	 *
	 * @param entities the collection remove.
	 */
	void remove(Collection<E> entities);

	/**
	 * find Entity instances.
	 *
	 * @param queryString
	 * @param values
	 * @return Collection all filtered objects.
	 */
	Collection<E> find(String queryString, Object[] values);

	/**
	 * findByExample Entity instances.
	 *
	 * @return Collection all filtered objects.
	 * @param entity
	 *            the identifier
	 */
	Collection<E> findByExample(E entity);

	/**
	 * findByExample Entity instances.
	 *
	 * @return Collection all filtered objects.
	 * @param entity
	 *            the identifier
	 */
	Collection<E> findByCriteria(DetachedCriteria criteria);

	/**
	 * Obtain a criteria for the entity class
	 *
	 * @return a criteria for the entity class.
	 */
	DetachedCriteria getCriteria();

	/**
	 * Return all objects related to the implementation of this DAO with no filter and ordered by orders.
	 *
	 * @return Collection all non filtered objects.
	 */
	Collection<E> loadAllOrderBy(Order... orders);

	/**
	 * Return all objects related to the implementation of this DAO with no filter, ordered by propertyName and ascending.
	 *
	 * @return Collection all non filtered objects.
	 */
	Collection<E> loadAllOrderby(String propertyName, Boolean ascending);

	/**
	 * Contruct order by from entityName, query and Order{@link Order}
	 *
	 * @param entityName
	 *            the entity name
	 * @param query
	 *            the query
	 * @param orders
	 *            the orders
	 * @return the resulted orderby
	 */
	String getQueryOrderedBy(String entityName, String query, Order... orders);
}
