/**
 *
 */
package com.ferri.persistence.dao.commons;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

import com.ferri.core.utils.ClazzUtils;
import com.ferri.core.utils.exception.GenericClassNotFound;
import com.ferri.core.utils.exception.GenericTypeNotFound;
import com.ferri.core.vo.commons.BaseVO;

/**
 * @author dferri
 */
public abstract class GenericBaseDAOImpl<E extends BaseVO, PK extends Serializable> extends HibernateDaoSupport implements GenericBaseDAO<E, PK> {

	protected Logger logger = Logger.getLogger(getClass());

	private Class<E> internalClassType = null;

	/**
	 * Specify a Class entity.
	 *
	 * @return a Class entity.
	 */
	public Class<E> getEntityClass() {
		return internalClassType;
	}

	/**
	 * @throws GenericClassNotFound
	 *             if the given class dont have the expected parent class
	 * @throws GenericTypeNotFound
	 *             if the given type is not found
	 */
	@SuppressWarnings("unchecked")
	private void initiateEntityType() throws GenericClassNotFound, GenericTypeNotFound {
		if (getEntityClass() == null) {
			internalClassType = (Class<E>) ClazzUtils.getEntityType(this.getClass(), GenericBaseDAOImpl.class);
		}
	}

	/**
	 * @throws GenericClassNotFound
	 *             if the given class dont have the expected parent class
	 * @throws GenericTypeNotFound
	 *             if the given type is not found
	 * @see com.ferri.core.behaviors.Initializable#initialize()
	 */
	public void initialize() throws GenericClassNotFound, GenericTypeNotFound {
		initiateEntityType();
	}

	/**
	 * @return Return the Entity Class Name.
	 */
	protected String getEntityClassName() {
		return getEntityClass().getName();
	}

	/**
	 * Append the message after the Entity Class Name.
	 *
	 * @param message
	 *            the message to append.
	 * @return Return the Entity Class Name with the message appended.
	 */
	protected String headEntityClassNameTo(String message) {
		return getEntityClass().getName() + message;
	}

	public DetachedCriteria getCriteria() {
		return DetachedCriteria.forClass(getEntityClass());
	}

	/**
	 * Contruct order by from entityName, query and Order{@link Order}
	 *
	 * @param entityName
	 *            the entity name
	 * @param query
	 *            the query
	 * @param orders
	 *            the orders
	 * @return the resulted orderby
	 */
	public String getQueryOrderedBy(String entityName, String query, Order... orders) {

		if (entityName == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".getOrderBy - 'entityName' can not be null"));
		}

		if (query == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".getOrderBy - 'query' can not be null"));
		}

		if (orders == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".getOrderBy - 'orders' can not be null"));
		}

		StringBuffer orderBuffer = new StringBuffer();

		if (orders != null && orders.length > 0) {

			orderBuffer.append(ORDER_BY);

			for (Order order : orders) {
				orderBuffer.append(entityName).append(SEPARATOR_DOT).append(order.toString()).append(SEPARATOR_COMMA);
			}

			orderBuffer.deleteCharAt(orderBuffer.length() - SEPARATOR_COMMA.length());
			orderBuffer.insert(0, query);
		}

		return orderBuffer.toString();
	}

	@SuppressWarnings("unchecked")
	public E load(PK key) {
		if (key == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".load - 'key' can not be null"));
		}

		final E entity = (E) getHibernateTemplate().load(getEntityClass(), key);
		return entity;
	}

	@SuppressWarnings("unchecked")
	public Collection<E> loadAll() {
		final Collection<E> results = getHibernateTemplate().loadAll(getEntityClass());
		return results;
	}

	public Collection<E> loadAllOrderby(String propertyName, Boolean ascending) {
		if (propertyName == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".loadAllOrderby - 'propertyName' can not be null"));
		}
		return loadAllOrderBy((ascending != null && ascending) ? Order.asc(propertyName) : Order.desc(propertyName));
	}

	public Collection<E> loadAllOrderBy(Order... orders) {
		if (orders == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".loadAllOrderby - 'orders' can not be null"));
		}

		DetachedCriteria criteria = getCriteria();

		for (Order order : orders) {
			criteria.addOrder(order);
		}

		return findByCriteria(criteria);
	}

	@SuppressWarnings("unchecked")
	public Collection<E> findByExample(E entity) {
		if (entity == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".findByExample - 'key' can not be null"));
		}

		final Collection<E> result = getHibernateTemplate().findByExample(entity);
		return result;
	}

	@SuppressWarnings("unchecked")
	public Collection<E> find(String queryString, Object[] values) {
		if (queryString == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".find - 'queryString' can not be null"));
		}

		final Collection<E> result = getHibernateTemplate().find(queryString, values);
		return result;
	}

	@SuppressWarnings("unchecked")
	public Collection<E> findByCriteria(DetachedCriteria criteria) {
		if (criteria == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".findByCriteria - 'criteria' can not be null"));
		}

		final Collection<E> result = getHibernateTemplate().findByCriteria(criteria);
		return result;
	}

	@SuppressWarnings("unchecked")
	public PK create(E entity) {
		if (entity == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".create - 'entity' can not be null"));
		}

		return (PK) getHibernateTemplate().save(entity);
	}

	public Collection<E> create(final Collection<E> entities) {
		if (entities == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".create - 'entities' can not be null"));
		}

		getHibernateTemplate().execute(new HibernateCallback() {
			public Object doInHibernate(Session session) throws HibernateException {
				for (Iterator<E> entityIterator = entities.iterator(); entityIterator.hasNext();) {
					create(entityIterator.next());
				}
				return null;
			}
		}, true);

		return entities;
	}

	public void saveOrUpdate(E entity) {
		if (entity == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".saveOrUpdate - 'entity' can not be null"));
		}

		getHibernateTemplate().saveOrUpdate(entity);
	}

	public void saveOrUpdate(final Collection<E> entities) {
		if (entities == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".saveOrUpdate - 'entities' can not be null"));
		}

		getHibernateTemplate().execute(new HibernateCallback() {
			public Object doInHibernate(Session session) throws HibernateException {
				for (Iterator<E> entityIterator = entities.iterator(); entityIterator.hasNext();) {
					saveOrUpdate(entityIterator.next());
				}
				return null;
			}
		}, true);
	}

	public void save(E entity) {
		if (entity == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".saveOrUpdate - 'entity' can not be null"));
		}

		getHibernateTemplate().save(entity);
	}

	public void update(E entity) {
		if (entity == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".update - 'entity' can not be null"));
		}

		getHibernateTemplate().update(entity);
	}

	public void removeByPk(PK id) {
		if (id == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".remove - 'id' can not be null"));
		}

		E entity = this.load(id);

		if (entity != null) {
			this.remove(entity);
		}
	}

	public void remove(E entity) {
		if (entity == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".remove - 'entity' can not be null"));
		}

		getHibernateTemplate().delete(entity);
	}

	public void remove(Collection<E> entities) {
		if (entities == null) {
			throw new IllegalArgumentException(headEntityClassNameTo(".remove - 'entities' can not be null"));
		}

		getHibernateTemplate().deleteAll(entities);
	}
}
