package com.ferri.portlet.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;


/**
 * Classe on deleguen les validacions del camps dels formularis jsp de manteniment i llstats.
 * @author dferri
 *
 */
public class ValidaCamps {

	private static final String ERROR_NULL	 		= "manteniments.error.field.obligatorio";
	private static final String ERROR_LONGITUD 		= "manteniments.error.field.longitud";
	private static final String ERROR_NUMERO		= "manteniments.error.field.numero";
	private static final String ERROR_FECHA			= "manteniments.error.field.fecha";
	private static final String ERROR_MES_ANY		= "manteniments.error.field.mesAny";	
	private static final String ERROR_FORMATO_NIF	= "manteniments.error.field.nif";
	private static final String ERROR_FORMATO_MAIL	= "manteniments.error.field.mail";

	/**
	 * Valida si un camp es null o buit
	 * @param camp
	 * @param errors
	 * @return boolean resultat
	 */
	public static Boolean valNull(Object camp, ArrayList<Missatge> errors){		
		if(camp == null || camp.toString().trim().equals("")){
			errors.add(new Missatge(Missatge.TYPE_ERROR, ERROR_NULL));
			return false;
		}
		return true;
	}
	
	/**
	 * Valida la longitud mnima
	 * @param name
	 * @param camp
	 * @param size
	 * @param errors
	 * @return boolean resultat
	 */
	public static Boolean valMinSize(String name, String camp, Integer size, ArrayList<Missatge> errors){		
		if(camp != null && !camp.trim().equals("")){
			if(camp.trim().length()< size){
				String[] args = {name, size.toString()};
				errors.add(new Missatge(Missatge.TYPE_ERROR, ERROR_LONGITUD, args));
				return false;
			}			
		}
		return true;
	}
	
	/**
	 * Valida que el camp sigui numric
	 * @param name
	 * @param camp
	 * @param errors
	 * @return boolean resultat
	 */
	public static Boolean valNumber(String name, String camp, ArrayList<Missatge> errors){
		if(camp != null && !camp.trim().equals("")){
			boolean ok = false;
			camp = camp.replaceAll(",", ".");
			try{
				Long.parseLong(camp.trim());
				ok = true;
			}catch(NumberFormatException e){}
			try{
				Double.parseDouble(camp.trim());
				ok = true;
			}catch(NumberFormatException e){}
			
			if(!ok){
				String args[] = {name};
				errors.add(new Missatge(Missatge.TYPE_ERROR, ERROR_NUMERO,args));
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Valida que sigui un format data dd/mm/aaaa
	 * @param camp
	 * @param errors
	 * @return boolean resultat
	 */
	public static Boolean valFecha(String camp, ArrayList<Missatge> errors){
		if(camp != null && !camp.trim().equals("")){
			SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy");
			try {
				df.parse(camp.trim());
			} catch (ParseException e) {
				errors.add(new Missatge(Missatge.TYPE_ERROR, ERROR_FECHA));
				return false;
			}			
		}
		return true;
	}

	/**
	 * Valida que sigui un format mm/aaaa
	 * @param camp
	 * @param errors
	 * @return boolean resultat
	 */
	public static Boolean valMesAny(String camp, ArrayList<Missatge> errors){
		if(camp != null && !camp.trim().equals("")){
			SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy");
			try {
				df.parse(Format.DEFAULT_DIA + camp.trim());
			} catch (ParseException e) {
				errors.add(new Missatge(Missatge.TYPE_ERROR, ERROR_MES_ANY));
				return false;
			}			
		}
		return true;
	}
	
	/**
	 * Valida que tingui una forma correcte de email aproximada, no veridica que existeixi.
	 * @param camp
	 * @param errors
	 * @return boolean resultat
	 */
	public static Boolean valMail(String camp, ArrayList<Missatge> errors){
		if(camp != null && !camp.trim().equals("")){
			
			//LA mejor forma sera comprovar si existe realmente, pero el usuario se ha de cercionar, adems recive los delivery status en su cuenta al enviar...
			if(	camp.indexOf("@") == camp.lastIndexOf("@") &&
				camp.indexOf("@")>0 &&
				camp.indexOf("@")+2 < camp.lastIndexOf(".")	&&
				camp.lastIndexOf(".")+2 < camp.length() &&
				camp.length() > 4){
				//ok
			}else{
				errors.add(new Missatge(Missatge.TYPE_ERROR, ERROR_FORMATO_MAIL));
				return false;
			}				
		}
		return true;
	}
	
	
	/**
	 * Valida que sigui un nif, cif, nie amb la forma correcte.
	 * @param camp
	 * @param errors
	 * @return boolean resultat
	 */
	public static Boolean valNIFCIF(String camp, ArrayList<Missatge> errors){
		if(camp != null && !camp.trim().equals("")){
			
			if(!esNifCifNieValido(camp)){
				errors.add(new Missatge(Missatge.TYPE_ERROR, ERROR_FORMATO_NIF));
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Valida cualquier nif cif nie espaol valido replicando los algoritmos de clculo de letras.
	 * @param numero identificativo
	 * @return bolean con el resultado
	 */
	private static final boolean esNifCifNieValido(String numero){
		final int TIPO_NIF = 1;
		final int TIPO_NIE = 2;
		final int TIPO_CIF = 3;
		final String NIF_STRING_ASOCIATION = "TRWAGMYFPDXBNJZSQVHLCKE";
		
		if(	numero == null 						|| 
			numero.trim().equals("")) {
			return false;
		}
		numero = numero.trim().toUpperCase();
		
		if(numero.length() != 9) {
			return false;
		}
		
		String control 	= numero.substring(8, 9);
		String n1	   	= numero.substring(0, 1);		
		String nResto 	= numero.substring(1, 8);
		
		String nie1		= "0";
		
		try {
			Integer.parseInt(nResto);
		}catch(NumberFormatException e) {
			return false;
		}
		
		int tipo_documento;		
		switch(n1.toCharArray()[0]) {
			case 'X': 
				nie1 = "0";
				//NIE
				tipo_documento = TIPO_NIE;
				break;
				
			case 'Y':
				nie1 = "1";
				//NIE
				tipo_documento = TIPO_NIE;
				break;
				
			case 'Z':
				nie1 = "2";
				//NIE
				tipo_documento = TIPO_NIE;
				break;

			case 'A': case 'B':	case 'C': case 'D': case 'E': case 'F': case 'G': case 'H': 
			case 'K': case 'L': case 'M': case 'N': case 'P': case 'Q':	case 'S':
				//CIF
				tipo_documento = TIPO_CIF;
				break;
				
			case '0': case '1':	case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
				//NIF
				tipo_documento = TIPO_NIF;
			break;
			
			default:
				return false;
		}
		
		try {
			
			if(tipo_documento == TIPO_NIF) {
				if(numero.equals(n1 + nResto + NIF_STRING_ASOCIATION.charAt((Integer.parseInt(n1 + nResto)% 23)))) {
					return true;
				}else{
					return false;
				}
				
			} else if(tipo_documento == TIPO_NIE) {
				if(numero.equals(n1 + nResto + NIF_STRING_ASOCIATION.charAt((Integer.parseInt(nie1 + nResto)% 23)))) {
					return true;
				}else{
					return false;
				}
			} else if(tipo_documento == TIPO_CIF) {
				int calculo[] = {0,2,4,6,8,1,3,5,7,9};
				int s = calculo[Integer.parseInt(nResto.substring(0,1))]
				        + Integer.parseInt(nResto.substring(1,2))
				        + calculo[Integer.parseInt(nResto.substring(2,3))]
				        + Integer.parseInt(nResto.substring(3,4))
				        + calculo[Integer.parseInt(nResto.substring(4,5))]
						+ Integer.parseInt(nResto.substring(5,6))
						+ calculo[Integer.parseInt(nResto.substring(6,7))];
				int d = ((10 -(s%10))%10);
				
				String corresp[] = {"J", "A", "B", "C", "D", "E", "F", "G", "H", "I"};
				if(control.equals(String.valueOf(d)) || control.equals(corresp[d])) {
					return true;
				}else {
					return false;
				}
			}
		
		}catch(Exception e) {
			return false;
		}
		return false;
	}
	
}
